@@ -26,14 +26,14 @@ def tag_list_equal():
 
 def find_pids():
     with LogixDriver('10.61.50.4/10') as plc:
-        pid_tags = []
 
-        for tag, _def in plc.tags.items():
-            # PIDs are structures, the data_type attribute will be a dict with data type definition.
-            # For tag types of 'atomic' the data type will a string, we need to skip those first.
-            # Then we can just look for tags whose data type name matches 'PID'
-            if _def['tag_type'] == 'struct':
-                if _def['data_type']['name'] == 'PID':
-                    pid_tags.append(tag)
+        # PIDs are structures, the data_type attribute will be a dict with data type definition.
+        # For tag types of 'atomic' the data type will a string, we need to skip those first.
+        # Then we can just look for tags whose data type name matches 'PID'
+        pid_tags = [
+            tag
+            for tag, _def in plc.tags.items()
+            if _def['tag_type'] == 'struct' and _def['data_type']['name'] == 'PID'
+        ]
 
-        print(pid_tags)
\ No newline at end of file
+        print(pid_tags)
